import  unittest
import warnings

from AnyQt.QtCore import QStringListModel
from AnyQt.QtWidgets import QApplication, QTableView, QStyledItemDelegate, \
    QStyleOptionViewItem


class Test_QAbstractItemView_itemDelegate(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        app = QApplication.instance()
        if app is None:
            app = QApplication([])
        cls.app = app

    @classmethod
    def tearDownClass(cls) -> None:
        cls.app = None
        super().tearDownClass()

    def test_qabstratitemview_item_delegate_for_index(self):
        view = QTableView()
        model = QStringListModel()
        model.setStringList(["AA", "BB"])
        view.setModel(model)
        idx1 = model.index(0, 0)
        idx2 = model.index(1, 0)
        delegate = QStyledItemDelegate()
        view.setItemDelegate(delegate)
        with warnings.catch_warnings(record=True):
            self.assertIs(view.itemDelegate(idx1), delegate)
            self.assertIs(view.itemDelegate(idx2), delegate)

    def test_qabstractitemview_view_options(self):
        view = QTableView()
        with warnings.catch_warnings(record=True):
            opt1 = view.viewOptions()
        self.assertIs(opt1.widget, view)
        self.assertTrue(opt1.showDecorationSelected)
        opt2 = QStyleOptionViewItem()
        view.initViewItemOption(opt2)
        self.assertIs(opt2.widget, view)
        self.assertTrue(opt2.showDecorationSelected)




import unittest

import AnyQt
from AnyQt.QtCore import Qt, QPointF, QRectF

from AnyQt.QtGui import QPainter, QPixmap, QImage, QGuiApplication, QColor
from AnyQt.QtWidgets import QApplication


class TestDrawPixmapFragments(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        app = QApplication.instance()
        if app is None:
            app = QApplication([])
        cls.app = app

    @classmethod
    def tearDownClass(cls) -> None:
        cls.app = None
        super().tearDownClass()

    def test(self):
        img = QImage(100, 100, QImage.Format_ARGB32)
        img.fill(Qt.green)
        p = QPainter(img)
        pix = QPixmap(10, 10)
        pix.fill(Qt.red)
        frgmts = [
            QPainter.PixmapFragment.create(
                QPointF(25, 25),
                QRectF(0, 0, 10, 10),
                5., 5.,
            ),
            QPainter.PixmapFragment.create(
                QPointF(75, 75),
                QRectF(0, 0, 10, 10),
                5., 5.,
            )

        ]
        p.drawPixmapFragments(frgmts, pix)
        p.end()
        self.assertEqual(QColor(img.pixel(10, 10)), QColor(Qt.red))
        self.assertEqual(QColor(img.pixel(80, 80)),  QColor(Qt.red))
        self.assertEqual(QColor(img.pixel(90, 10)), QColor(Qt.green))
        self.assertEqual(QColor(img.pixel(10, 90)), QColor(Qt.green))

    @unittest.skipIf(not AnyQt.USED_API.lower().startswith("pyside"), "PySide only")
    def test_pyside(self):
        img = QImage(100, 100, QImage.Format_ARGB32)
        img.fill(Qt.green)
        p = QPainter(img)
        pix = QPixmap(10, 10)
        pix.fill(Qt.red)
        frgmt = QPainter.PixmapFragment.create(
            QPointF(25, 25),
            QRectF(0, 0, 10, 10),
            5., 5.,
        )
        p.drawPixmapFragments(frgmt, 1, pix)
        p.end()
        self.assertEqual(QColor(img.pixel(10, 10)), QColor(Qt.red))


import unittest

class TestExports(unittest.TestCase):
    names = [
        "AnyQt.QtCore:Signal",
        "AnyQt.QtCore:Slot",
        "AnyQt.QtCore:Property",

        "AnyQt.QtGui:QUndoCommand",
        "AnyQt.QtGui:QUndoGroup",
        "AnyQt.QtGui:QUndoStack",
        "AnyQt.QtGui:QShortcut",
        "AnyQt.QtGui:QAction",
        "AnyQt.QtGui:QActionGroup",
        "AnyQt.QtGui:QFileSystemModel",
    ]
    def test_exports(self):
        for name in self.names:
            pkg, _, item = name.rpartition(":")
            module = __import__(pkg, fromlist=[item])
            getattr(module, item)


import unittest
from itertools import count

from AnyQt.QtCore import QObject, isdeleted
from AnyQt.QtWidgets import QApplication
from AnyQt.QtTest import QTest


class TestQtTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        app = QApplication.instance()
        if app is None:
            app = QApplication([])
        cls.app = app

    @classmethod
    def tearDownClass(cls) -> None:
        cls.app = None
        super().tearDownClass()

    def test_qwait(self):
        obj = QObject()
        obj.deleteLater()
        QTest.qWait(0)
        self.assertTrue(isdeleted(obj))

    def test_qwaitfor(self):
        counter = count()
        current = 0

        def pred():
            nonlocal current
            current = next(counter)
            return current > 4
        self.assertTrue(QTest.qWaitFor(pred, 100000))
        self.assertTrue(current == 5)
        self.assertFalse(QTest.qWaitFor(lambda: False, 10))



import unittest
import gc

from AnyQt.QtGui import QStandardItem, QStandardItemModel

class TestQStandardItem(unittest.TestCase):
    def test(self):
        model = QStandardItemModel()
        item_parent = QStandardItem("parent")
        item_child = QStandardItem("child")
        model.insertRow(0, item_parent)
        item_parent.insertRow(0, item_child)
        self.assertEqual(model.index(0, 0).data(), "parent")
        self.assertEqual(model.index(0, 0, model.index(0, 0)).data(), "child")
        del item_child
        del item_parent
        gc.collect()
        self.assertEqual(model.index(0, 0).data(), "parent")
        self.assertEqual(model.index(0, 0, model.index(0, 0)).data(), "child")



import unittest

from weakref import ref

from AnyQt.QtWidgets import QMenu, QApplication
from AnyQt.QtGui import QAction


class TestQAction_setMenu(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        app = QApplication.instance()
        if app is None:
            app = QApplication([])
        cls.app = app

    @classmethod
    def tearDownClass(cls) -> None:
        cls.app = None
        super().tearDownClass()

    def test(self):
        ac = QAction()
        menu = QMenu()
        wref = ref(menu)
        ac.setMenu(menu)
        self.assertIs(ac.menu(), menu)
        ac.setMenu(None)
        self.assertIs(ac.menu(), None)
        menu.setParent(None)  # parent is None but without this PySide2 fails??
        del menu
        self.assertIsNone(wref())



import  unittest

from AnyQt.QtCore import QBuffer, QSizeF

from AnyQt.QtGui import QPdfWriter, QPageSize


class TestQPDFWriter(unittest.TestCase):
    def test(self):
        b = QBuffer()
        w = QPdfWriter(b)
        size = QPageSize(QSizeF(10, 10), QPageSize.Millimeter)
        _ = w.setPageSize(size)
        self.assertTrue(w.setPageSize(size))


#! /usr/bin/env python
# encoding: utf-8
import io

from setuptools import setup, find_packages

NAME = "AnyQt"
VERSION = "0.2.1"
AUTHOR = "Aleš Erjavec"
AUTHOR_EMAIL = "ales.erjavec@fri.uni-lj.si"
URL = "https://github.com/ales-erjavec/anyqt"
PACKAGES = find_packages(".", include="AnyQt*")

DESCRIPTION = "PyQt5/PyQt6 compatibility layer."

with io.open("README.txt", encoding="utf-8") as f:
    README = f.read()

LICENSE = "GPLv3"

CLASSIFIERS = [
    "Development Status :: 1 - Planning",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 3",
]

KEYWORDS = [
    "GUI", "PyQt4", "PyQt5", "PySide", "PySide2", "compatibility"
]

PROJECT_URLS = {
    "Bug Reports": "https://github.com/ales-erjavec/anyqt/issues",
    "Source": URL,
    "Documentation": "https://anyqt.readthedocs.io/en/stable/"
}

if __name__ == "__main__":
    setup(
        name=NAME,
        version=VERSION,
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        url=URL,
        description=DESCRIPTION,
        long_description=README,
        license=LICENSE,
        keywords=KEYWORDS,
        classifiers=CLASSIFIERS,
        packages=PACKAGES,
        project_urls=PROJECT_URLS,
        python_requires=">=3.6",
    )



from typing import Union
from functools import lru_cache

from AnyQt.QtCore import QObject, Property


@lru_cache(maxsize=200)
def _converter(type_: Union[type, str]):
    class Obj(QObject):
        _f = None
        def _set(self, val):
            self._f = val
        def _get(self):
            return self._f

        prop = Property(type_, _get, _set)

    def convert(value):
        inst = Obj()
        ok = inst.setProperty('prop', value)
        if ok:
            return inst.property('prop')
        else:
            return None
    return convert


def qvariant_cast(value, type_: Union[type, str]):
    converter = _converter(type_)
    return converter(value)



import unittest

from AnyQt.QtGui import QAction
from AnyQt.QtWidgets import QWidget, QApplication


class TestQActionEvent_action(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        app = QApplication.instance()
        if app is None:
            app = QApplication([])
        cls.app = app

    @classmethod
    def tearDownClass(cls) -> None:
        cls.app = None
        super().tearDownClass()

    def test_action(self):
        a = QAction()
        b = QAction()

        class Widget(QWidget):
            def test(self, event):
                pass

            def actionEvent(self, event):
                super().actionEvent(event)
                self.test(event)

        widget = Widget()

        def test(ev):
            assert ev.action() is b
        widget.test = test
        widget.addAction(b)

        def test(ev):
            assert ev.action() is a
            assert ev.before() is b
        widget.test = test
        widget.insertAction(b, a)
        del widget.test



import subprocess
import sys

apis = {
    'pyqt4': 'PyQt4',
    'pyqt5': 'PyQt5',
    'pyside': 'PySide',
    'pyside2': 'PySide2',
    'pyqt6:': "PyQt6"
}

submodules_common = [
    'QtCore',
    'QtGui',
    'QtHelp',
    'QtMultimedia',
    'QtNetwork',
    'QtOpenGL',
    'QtPrintSupport',
    'QtSql',
    'QtSvg',
    'QtTest',
    'QtWidgets',
    'QtXml',

]
submodules_qt4 = [
    'QtXmlPatterns'
]
submodules_qt5 = [
    'QtMultimediaWidgets',
    'QtWebChannel',
    'QtWebEngineWidgets',
    'QtWebEngineCore',
    'QtWebSockets',
    'QtQml',
    'QtXmlPatterns',
]

submodules_qt6 = [
    "QtSvgWidgets",
]


def get_qt_version(modname):
    cmd = [sys.executable, '-c', 'import '+modname+';from AnyQt.QtCore import QT_VERSION_STR; print(QT_VERSION_STR)']
    return subprocess.check_output(cmd)


def try_import(modname):
    cmd = [sys.executable, '-c', 'import '+modname]
    ret = subprocess.call(cmd)
    return ret == 0


def try_import_anyqt(modname, submodule):
    cmd = [sys.executable, '-c', 'import '+modname+'; import AnyQt.'+submodule]
    ret = subprocess.call(cmd)
    return ret == 0


for api in apis:
    modname = apis[api]
    if try_import(modname):
        qt_version = get_qt_version(modname).decode()
        print('-- Found', modname, 'Qt=', qt_version)
        submodules = list(submodules_common)
        if qt_version[0] == '4':
            submodules.extend(submodules_qt4)
        if qt_version[0] == '5':
            submodules.extend(submodules_qt5)
        if qt_version[0] == '6':
            submodules.extend(submodules_qt6)
        for submodule in submodules:
            if try_import_anyqt(modname, submodule):
                print(submodule, 'Ok')
            else:
                print(submodule, 'FAIL')
    else:
        print(modname, '-- not found')
    




import unittest

from AnyQt.QtGui import QFontDatabase
from AnyQt.QtWidgets import QApplication


class TestQFontDatabase(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        app = QApplication.instance()
        if app is None:
            app = QApplication([])
        cls.app = app

    @classmethod
    def tearDownClass(cls) -> None:
        # cls.app = None
        super().tearDownClass()

    def test_qfontdatabase_static(self):
        families = QFontDatabase.families()
        styles = QFontDatabase.styles(families[0])
        systems = QFontDatabase.writingSystems()
        systems = QFontDatabase.writingSystems(families[0])



import unittest

from AnyQt.QtCore import QSettings, QStandardPaths


class TestQSettings(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()
        QStandardPaths.setTestModeEnabled(True)
        QSettings.setDefaultFormat(QSettings.IniFormat)

    def tearDown(self) -> None:
        QStandardPaths.setTestModeEnabled(False)
        QSettings.setDefaultFormat(QSettings.NativeFormat)
        super().tearDown()

    def test_qsettings(self):
        s = QSettings()
        s.setValue("one", 1)
        s.setValue("one-half", 0.5)
        s.setValue("empty", "")
        s.setValue("true", True)
        s.sync()
        del s
        s = QSettings()
        self.assertEqual(s.value("one", type=int), 1)
        self.assertEqual(s.value("one-half", type=float), 0.5)
        self.assertEqual(s.value("empty", type=str), "")
        self.assertEqual(s.value("true", type=bool), True)